/*

-Procedure turnrates ( From a time series of orientations, 
                       return angular velocities and accelerations. ) 

   Jeff Bytof, last update 15-May-2001

*/

#include "SpiceUsr.h"
#include "SpiceZmc.h"
#include "SpiceZfc.h"

void turnrates ( SpiceInt         ROOM, 
                 SpiceInt         nq, 
                 SpiceDouble      t         [ROOM],
                 SpiceDouble      rot       [ROOM][3][3],
                 SpiceInt       * nv,
                 SpiceDouble      tv        [ROOM],
                 SpiceDouble      angvel    [ROOM][3],
                 SpiceInt       * na,
                 SpiceDouble      ta        [ROOM],
                 SpiceDouble      angacc    [ROOM][3]
               )

{

/*

-Brief_I/O

    Variable    I/O      Description
    --------    ---      -----------
    ROOM         I       Maximum size of output arrays.
    nq           I       Number of input orientations.
    t            I       Orientation time array, in seconds.
    rot          I       Orientation matrix time series.
    nv           O       Number of angular velocity vectors.
    tv           O       Array of angular velocity times.
    angvec       O       Angular velocity time series.
    na           O       Number of angular velocity vectors.
    ta           O       Array of angular velocity times.
    angacc       O       Angular velocity time series.

*/
   
/*
   Local variables.
*/


   SpiceDouble      angle; 
   SpiceDouble      axis     [3];
   SpiceDouble      h;
   SpiceDouble      infrot   [3][3];
   SpiceDouble      temp     [3]; 
   SpiceDouble      vrate; 

   SpiceInt         i;  


/* 
   Determine the angular velocity vector and angular rate
   between each orientation
*/
 
   *nv = nq - 1;

   for ( i=0; i<*nv; i++ ) { 
 
      mxmt_c ( rot[i], rot[i+1], infrot );

      raxisa_c ( infrot, axis, &angle );

      tv[i] = (t[i] + t[i+1]) / 2.0;

      h = t[i+1] - t[i];

      vrate = angle / h;

      vscl_c ( angle/h, axis, angvel[i] );
  
   }

/*
   Determine the angular acceleration from the difference
   of two angular velocity vectors.
*/

   *na = *nv - 1;

   for ( i=0; i<*na; i++ ) { 
 
      h = tv[i+1] - tv[i];

      ta[i] = (tv[i] + tv[i+1]) / 2.0;


      vsub_c ( angvel[i+1], angvel[i], temp );

      vscl_c ( 1.0/h, temp, angacc[i] );

   }

   return;

}

